package junit.framework;

import java.util.*;
import java.lang.reflect.*;

/**
 */
public class TestCaseBlock extends Object {
	protected Vector fTests;
	protected Class fExceptionClass;
	protected String fMessage;
public TestCaseBlock() {
	this("expected exception not thrown");
}
public TestCaseBlock(String msg) {
	fTests= new Vector();
	fMessage= msg;
}
/*
 * Fail with given failure message.
 * Copied from TestCase.
 */
public void fail(String message) {
	throw new AssertionFailedError(message);
}
/*
 * Check for method duplicates.
 * Copied from TestCase.
 */
protected boolean hasTestMethod(Method newMethod) {
	for (Enumeration i= fTests.elements(); i.hasMoreElements(); ) {
		Method m= (Method) i.nextElement();
		String name= m.getName();
		if (name.equals(newMethod.getName())) {
			return true;
		}
	}
	return false;
}
/*
 * Check method name for JUnit test method naming convention.
 * Copied from TestSuite.
 */
protected boolean isTestMethod(Method m) {
	String name= m.getName();
	Class[] parameters= m.getParameterTypes();
	Class returnType= m.getReturnType();
	
	return parameters.length == 0 &&
		name.startsWith("test") &&
		returnType.equals(Void.TYPE) &&
		Modifier.isPublic(m.getModifiers());
 }        
public String message() {
	return fMessage;
}
/*
 * Run all test methods of this Block class.
 */
public void run(Class expectedEC) {
	Vector tests= tests();
	for (Enumeration i= tests.elements(); i.hasMoreElements();) {
		Method m= (Method) i.nextElement();
		try {
			m.invoke(this, new Class[0]);
			fail(message());
		}
		catch (InvocationTargetException ite) {
			Class actualEC= ite.getTargetException().getClass();
			if (!expectedEC.isAssignableFrom(actualEC)) {
				fail(message());
			}
		}
		catch (Exception e) {
			// that's a classical cannot happen ...
		}
	}
}
/*
 * Determine test methods in this TestCaseBlock class
 */
public Vector tests() {
	if (fTests.size() != 0) {
		return fTests;
	}
	Class theClass= getClass();
	while (theClass != TestCaseBlock.class) {
		Method[] methods= theClass.getDeclaredMethods();
		for (int i= 0; i < methods.length; i++) {
			Method m= methods[i];
			String name= m.getName();
			if (isTestMethod(m) && !hasTestMethod(m)) {
				fTests.addElement(m);
			}
		}
		theClass= theClass.getSuperclass();
	}
	return fTests;
}
}
